/* ========================================================================
 *                              Barrier.cpp
 * ------------------------------------------------------------------------
 *  Implementation file for the Barrier class.
 *
 *  Copyright 1998 by G. Wade Johnson (Telescan, Inc.)
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include "Barrier.h"
#include "CriticalSection.h"
#include "Event.h"
#include <string>
#include <windows.h>


Barrier::Barrier( short num )
 : myMaxThreads(num), myNumThreads(0), myName(""), mypMutex(0), mypBarrier(0)
{
  mypMutex   = new CriticalSection;
  mypBarrier = new ManualResetEvent;

  mypBarrier->Reset();
}



Barrier::Barrier( short num, const char *name )
 : myMaxThreads(num), myNumThreads(0), myName(name), mypMutex(0), mypBarrier(0)
{
  mypMutex   = new CriticalSection;
  mypBarrier = new ManualResetEvent( myName.c_str() );

  mypBarrier->Reset();
}



Barrier::~Barrier()
{
  delete mypMutex;
  delete mypBarrier;
}


void   Barrier::Wait()
{
 {
  CriticalSectionLock  sync( *mypMutex );

   if(++myNumThreads >= myMaxThreads)
    {
     myNumThreads = 0;
     
     // we have enough, clear waiting threads
     mypBarrier->Pulse();
     return;
    }
  }

 mypBarrier->Wait();
}


void   Barrier::Release()
{
  CriticalSectionLock  sync( *mypMutex );

  myNumThreads = 0;
  mypBarrier->Pulse();
}
