/* ------------------------------------------------------------------------
 *                            TestBarrier.cpp
 * ------------------------------------------------------------------------
 *   Test driver for the Barrier synchronization class.
 *
 *  Copyright 1998 by G. Wade Johnson (Telescan, Inc.)
 *   Use of this code is placed in the public domain as long as the
 *   copyright notice is retained.
 */

#include <windows.h>
#include <iostream>
#include "Barrier.h"
#include "Event.h"
#include "CriticalSection.h";

const int NumThreads = 5;

Barrier             StartingLine( NumThreads );
CriticalSection     Console;
AutoResetEvent      LockStep;     // Used to reduce startup conflicts
int                 ThreadNum=0;  // I hate this, but it is simple.

DWORD  WINAPI   TestThread( LPVOID lpvB );

int   main( void )
 {
  HANDLE         Threads[NumThreads];
  DWORD          Dummy;
  int            i;

  LockStep.Reset();
  for(i=0;i < NumThreads;++i,++ThreadNum)
   {
    Threads[ThreadNum] = CreateThread( NULL, 0, TestThread, (LPVOID)&StartingLine,
                                       0, &Dummy );
    LockStep.Wait();
   }

  for(i=0;i < ThreadNum;++i)
   {
    WaitForSingleObject( Threads[i], INFINITE );
   }

  return 0;
 }



DWORD  WINAPI	TestThread( LPVOID lpvB )
 {
  Barrier   *pB = (Barrier*)lpvB;
  int        myNum = ThreadNum; // Using global data, even though I know it's bad.

   {
    CriticalSectionLock  sync(Console);
    cout << "Thread " << myNum << " is ready" << endl;
   }

  LockStep.Set();    // Release main thread
  pB->Wait();

  for(int i = 0;i <= myNum;++i)
   {
     {
      CriticalSectionLock  sync(Console);
      cout << myNum << ": Wasting time" << endl;
     }
    Sleep( 2000 );
   }
  pB->Wait();

   {
    CriticalSectionLock  sync(Console);
    cout << myNum << ": I'm finished!!" << endl;
   }
  Sleep( 2000 );

  return 0L;
 }
